package com.ustadmobile.core.db.dao

import com.ustadmobile.door.DoorDbType
import com.ustadmobile.door.EntityInsertionAdapter
import com.ustadmobile.door.PreparedStatementConfig
import com.ustadmobile.door.ext.prepareAndUseStatement
import com.ustadmobile.door.ext.prepareAndUseStatementAsync
import com.ustadmobile.door.jdbc.PreparedStatement
import com.ustadmobile.door.jdbc.ext.executeUpdateAsyncKmp
import com.ustadmobile.door.jdbc.ext.mapNextRow
import com.ustadmobile.door.jdbc.ext.mapRows
import com.ustadmobile.door.jdbc.ext.useResults
import com.ustadmobile.door.room.RoomDatabase
import com.ustadmobile.lib.db.entities.ContentCategorySchema
import kotlin.Boolean
import kotlin.Long
import kotlin.String
import kotlin.Unit
import kotlin.collections.List

public class ContentCategorySchemaDao_JdbcKt(
  public val _db: RoomDatabase,
) : ContentCategorySchemaDao() {
  public val _insertAdapterContentCategorySchema_: EntityInsertionAdapter<ContentCategorySchema> =
      object : EntityInsertionAdapter<ContentCategorySchema>(_db) {
    public override fun makeSql(returnsId: Boolean) =
        "INSERT INTO ContentCategorySchema (contentCategorySchemaUid, schemaName, schemaUrl, contentCategorySchemaLocalChangeSeqNum, contentCategorySchemaMasterChangeSeqNum, contentCategorySchemaLastChangedBy, contentCategorySchemaLct) VALUES(?, ?, ?, ?, ?, ?, ?)"

    public override fun bindPreparedStmtToEntity(stmt: PreparedStatement,
        entity: ContentCategorySchema): Unit {
      if(entity.contentCategorySchemaUid == 0L) {
        stmt.setObject(1, null)
      } else {
        stmt.setLong(1, entity.contentCategorySchemaUid)
      }
      stmt.setString(2, entity.schemaName)
      stmt.setString(3, entity.schemaUrl)
      stmt.setLong(4, entity.contentCategorySchemaLocalChangeSeqNum)
      stmt.setLong(5, entity.contentCategorySchemaMasterChangeSeqNum)
      stmt.setInt(6, entity.contentCategorySchemaLastChangedBy)
      stmt.setLong(7, entity.contentCategorySchemaLct)
    }
  }

  public override fun insert(entity: ContentCategorySchema): Long {
    val _retVal = _insertAdapterContentCategorySchema_.insertAndReturnId(entity)
    return _retVal
  }

  public override suspend fun insertAsync(entity: ContentCategorySchema): Long {
    val _retVal = _insertAdapterContentCategorySchema_.insertAndReturnIdAsync(entity)
    return _retVal
  }

  public override fun insertList(entityList: List<ContentCategorySchema>): Unit {
    _insertAdapterContentCategorySchema_.insertList(entityList)
  }

  public override fun updateList(entityList: List<ContentCategorySchema>): Unit {
    val _sql =
        "UPDATE ContentCategorySchema SET schemaName = ?, schemaUrl = ?, contentCategorySchemaLocalChangeSeqNum = ?, contentCategorySchemaMasterChangeSeqNum = ?, contentCategorySchemaLastChangedBy = ?, contentCategorySchemaLct = ? WHERE contentCategorySchemaUid = ?"
    _db.prepareAndUseStatement(_sql) {
       _stmt ->
      _stmt.getConnection().setAutoCommit(false)
      for(_entity in entityList) {
        _stmt.setString(1, _entity.schemaName)
        _stmt.setString(2, _entity.schemaUrl)
        _stmt.setLong(3, _entity.contentCategorySchemaLocalChangeSeqNum)
        _stmt.setLong(4, _entity.contentCategorySchemaMasterChangeSeqNum)
        _stmt.setInt(5, _entity.contentCategorySchemaLastChangedBy)
        _stmt.setLong(6, _entity.contentCategorySchemaLct)
        _stmt.setLong(7, _entity.contentCategorySchemaUid)
        _stmt.executeUpdate()
      }
      _stmt.getConnection().commit()
    }
  }

  public override fun update(entity: ContentCategorySchema): Unit {
    val _sql =
        "UPDATE ContentCategorySchema SET schemaName = ?, schemaUrl = ?, contentCategorySchemaLocalChangeSeqNum = ?, contentCategorySchemaMasterChangeSeqNum = ?, contentCategorySchemaLastChangedBy = ?, contentCategorySchemaLct = ? WHERE contentCategorySchemaUid = ?"
    _db.prepareAndUseStatement(_sql) {
       _stmt ->
      _stmt.setString(1, entity.schemaName)
      _stmt.setString(2, entity.schemaUrl)
      _stmt.setLong(3, entity.contentCategorySchemaLocalChangeSeqNum)
      _stmt.setLong(4, entity.contentCategorySchemaMasterChangeSeqNum)
      _stmt.setInt(5, entity.contentCategorySchemaLastChangedBy)
      _stmt.setLong(6, entity.contentCategorySchemaLct)
      _stmt.setLong(7, entity.contentCategorySchemaUid)
      _stmt.executeUpdate()
    }
  }

  public override suspend fun replicateOnNewNode(newNodeId: Long): Unit {
    _db.prepareAndUseStatementAsync(PreparedStatementConfig("""
    |
    |     REPLACE INTO ContentCategorySchemaReplicate(ccsPk, ccsDestination)
    |      SELECT DISTINCT ContentCategorySchema.contentCategorySchemaUid AS ccsPk,
    |             ? AS ccsDestination
    |        FROM ContentCategorySchema
    |       WHERE ContentCategorySchema.contentCategorySchemaLct != COALESCE(
    |             (SELECT ccsVersionId
    |                FROM ContentCategorySchemaReplicate
    |               WHERE ccsPk = ContentCategorySchema.contentCategorySchemaUid
    |                 AND ccsDestination = ?), 0) 
    |      /*psql ON CONFLICT(ccsPk, ccsDestination) DO UPDATE
    |             SET ccsPending = true
    |      */       
    |    
    """.trimMargin() , postgreSql = """
    |INSERT INTO ContentCategorySchemaReplicate(ccsPk, ccsDestination)
    |      SELECT DISTINCT ContentCategorySchema.contentCategorySchemaUid AS ccsPk,
    |             ? AS ccsDestination
    |        FROM ContentCategorySchema
    |       WHERE ContentCategorySchema.contentCategorySchemaLct != COALESCE(
    |             (SELECT ccsVersionId
    |                FROM ContentCategorySchemaReplicate
    |               WHERE ccsPk = ContentCategorySchema.contentCategorySchemaUid
    |                 AND ccsDestination = ?), 0) 
    |       ON CONFLICT(ccsPk, ccsDestination) DO UPDATE
    |             SET ccsPending = true
    |             
    |    
    |""".trimMargin())) { _stmt -> 
      _stmt.setLong(1,newNodeId)
      _stmt.setLong(2,newNodeId)
      _stmt.executeUpdateAsyncKmp()
    }
  }

  public override suspend fun replicateOnChange(): Unit {
    _db.prepareAndUseStatementAsync(PreparedStatementConfig("""
    |
    | REPLACE INTO ContentCategorySchemaReplicate(ccsPk, ccsDestination)
    |  SELECT DISTINCT ContentCategorySchema.contentCategorySchemaUid AS ccsUid,
    |         UserSession.usClientNodeId AS ccsDestination
    |    FROM ChangeLog
    |         JOIN ContentCategorySchema
    |             ON ChangeLog.chTableId = 2
    |                AND ChangeLog.chEntityPk = ContentCategorySchema.contentCategorySchemaUid
    |         JOIN UserSession ON UserSession.usStatus = 1
    |   WHERE UserSession.usClientNodeId != (
    |         SELECT nodeClientId 
    |           FROM SyncNode
    |          LIMIT 1)
    |     AND ContentCategorySchema.contentCategorySchemaLct != COALESCE(
    |         (SELECT ccsVersionId
    |            FROM ContentCategorySchemaReplicate
    |           WHERE ccsPk = ContentCategorySchema.contentCategorySchemaUid
    |             AND ccsDestination = UserSession.usClientNodeId), 0)
    | /*psql ON CONFLICT(ccsPk, ccsDestination) DO UPDATE
    |     SET ccsPending = true
    |  */               
    |    
    """.trimMargin() , postgreSql = """
    |INSERT INTO ContentCategorySchemaReplicate(ccsPk, ccsDestination)
    |  SELECT DISTINCT ContentCategorySchema.contentCategorySchemaUid AS ccsUid,
    |         UserSession.usClientNodeId AS ccsDestination
    |    FROM ChangeLog
    |         JOIN ContentCategorySchema
    |             ON ChangeLog.chTableId = 2
    |                AND ChangeLog.chEntityPk = ContentCategorySchema.contentCategorySchemaUid
    |         JOIN UserSession ON UserSession.usStatus = 1
    |   WHERE UserSession.usClientNodeId != (
    |         SELECT nodeClientId 
    |           FROM SyncNode
    |          LIMIT 1)
    |     AND ContentCategorySchema.contentCategorySchemaLct != COALESCE(
    |         (SELECT ccsVersionId
    |            FROM ContentCategorySchemaReplicate
    |           WHERE ccsPk = ContentCategorySchema.contentCategorySchemaUid
    |             AND ccsDestination = UserSession.usClientNodeId), 0)
    |  ON CONFLICT(ccsPk, ccsDestination) DO UPDATE
    |     SET ccsPending = true
    |                 
    |    
    |""".trimMargin())) { _stmt -> 
      _stmt.executeUpdateAsyncKmp()
    }
  }

  public override fun publicContentCategorySchemas(): List<ContentCategorySchema> =
      _db.prepareAndUseStatement(PreparedStatementConfig("SELECT ContentCategorySchema.* FROM ContentCategorySchema"
      )) { _stmt -> 
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapRows {
        val _tmp_contentCategorySchemaUid = _result.getLong("contentCategorySchemaUid")
        val _tmp_schemaName = _result.getString("schemaName")
        val _tmp_schemaUrl = _result.getString("schemaUrl")
        val _tmp_contentCategorySchemaLocalChangeSeqNum =
            _result.getLong("contentCategorySchemaLocalChangeSeqNum")
        val _tmp_contentCategorySchemaMasterChangeSeqNum =
            _result.getLong("contentCategorySchemaMasterChangeSeqNum")
        val _tmp_contentCategorySchemaLastChangedBy =
            _result.getInt("contentCategorySchemaLastChangedBy")
        val _tmp_contentCategorySchemaLct = _result.getLong("contentCategorySchemaLct")
        ContentCategorySchema().apply {
          this.contentCategorySchemaUid = _tmp_contentCategorySchemaUid
          this.schemaName = _tmp_schemaName
          this.schemaUrl = _tmp_schemaUrl
          this.contentCategorySchemaLocalChangeSeqNum = _tmp_contentCategorySchemaLocalChangeSeqNum
          this.contentCategorySchemaMasterChangeSeqNum =
              _tmp_contentCategorySchemaMasterChangeSeqNum
          this.contentCategorySchemaLastChangedBy = _tmp_contentCategorySchemaLastChangedBy
          this.contentCategorySchemaLct = _tmp_contentCategorySchemaLct
        }
      }
    }
  }

  public override fun findBySchemaUrl(schemaUrl: String): ContentCategorySchema? =
      _db.prepareAndUseStatement(PreparedStatementConfig("SELECT * FROM ContentCategorySchema WHERE schemaUrl = ?"
      )) { _stmt -> 
    _stmt.setString(1,schemaUrl)
    _stmt.executeQuery().useResults{ _result -> 
      _result.mapNextRow(null) {
        val _tmp_contentCategorySchemaUid = _result.getLong("contentCategorySchemaUid")
        val _tmp_schemaName = _result.getString("schemaName")
        val _tmp_schemaUrl = _result.getString("schemaUrl")
        val _tmp_contentCategorySchemaLocalChangeSeqNum =
            _result.getLong("contentCategorySchemaLocalChangeSeqNum")
        val _tmp_contentCategorySchemaMasterChangeSeqNum =
            _result.getLong("contentCategorySchemaMasterChangeSeqNum")
        val _tmp_contentCategorySchemaLastChangedBy =
            _result.getInt("contentCategorySchemaLastChangedBy")
        val _tmp_contentCategorySchemaLct = _result.getLong("contentCategorySchemaLct")
        ContentCategorySchema().apply {
          this.contentCategorySchemaUid = _tmp_contentCategorySchemaUid
          this.schemaName = _tmp_schemaName
          this.schemaUrl = _tmp_schemaUrl
          this.contentCategorySchemaLocalChangeSeqNum = _tmp_contentCategorySchemaLocalChangeSeqNum
          this.contentCategorySchemaMasterChangeSeqNum =
              _tmp_contentCategorySchemaMasterChangeSeqNum
          this.contentCategorySchemaLastChangedBy = _tmp_contentCategorySchemaLastChangedBy
          this.contentCategorySchemaLct = _tmp_contentCategorySchemaLct
        }
      }
    }
  }
}
